/*
 *  daterangeselector.js
 *  Author: Matthew Aho
 *  Github: ahomatthew/daterangselector
 *  Date: January 31, 2013
 *  Description:
 *      daterangeselector.js provides a date range selection plugin that actually works and looks nice. 
 *  Todo:
 *      - Allow the ability to pass in a string format for the clean date
 *      - Add listener for if user clicks document, the popup hides
 *      - Make it an actual jquery plugin, not sure how to go about doing that
 *      - Modularize the code in the appropriate way, obviously this isn't done right ;)
 *
 */

$(function() {
    $.daterangeselector = new Object();

    function isInt(n) {
        return typeof n === 'number' && n % 1 == 0;
    }

    // Initialise date range selector on target element
    $.daterangeselector.init = function(target) {

        // Initialise the target specific object if it doesn't already exist
        if(!$.daterangeselector[target]) {
            $.daterangeselector[target] = new Object();
        }

        // Create additional dom elements
        // Start element
        if(!$('#' + target + ' input.start').length || $('#' + target + ' input.start').length < 1) {
            $('#' + target).append('<input type="hidden" class="start" value="" />');
        }
        // End element
        if(!$('#' + target + ' input.end').length || $('#' + target + ' input.end').length < 1) {
            $('#' + target).append('<input type="hidden" class="end" value="" />');
        }
        // Popup element
        if(!$('#' + target + ' div.popup').length || $('#' + target + ' div.popup').length < 1) {
            $('#' + target).append('<div class="popup"><div class="closed"><button class="close">&times;</button></div><div class="content"></div></div>');
        }

        // Get dom elements and store them as variable for later use
        elems = new Object();
        elems.start = $("#" + target + " input.start");
        elems.end = $("#" + target + " input.end");
        elems.range = $("#" + target + " input.range");
        elems.popup = $("#" + target + " .popup");

        // Establish listeners for target
        if(!$.daterangeselector[target].initListener) {
            $.daterangeselector[target].initListener = true;
            $.daterangeselector.initListeners(elems, target);
        }

        // Check and see if the range has been set yet, if not -- set it
        if(!$.daterangeselector[target].range) {
            $.daterangeselector[target].range = new Object();
        }

        if(elems.range.attr('start') && elems.range.attr('end')) {
            if(moment(elems.range.attr('start')).isValid() && moment(elems.range.attr('end')).isValid()) {
                $.daterangeselector[target].range.start = elems.range.attr('start');
                $.daterangeselector[target].range.end = elems.range.attr('end');
            }
        }

        // If no date is selected, lets set the current date to today?
        if(!$.daterangeselector[target].range.start) {
            $.daterangeselector[target].range.start = moment().format("YYYY-MM-DD");
        }

        // If the start is set, but the end is not, go ahead and set it to the start
        if($.daterangeselector[target].range.start && !$.daterangeselector[target].range.end) {
            $.daterangeselector[target].range.end = $.daterangeselector[target].range.start;
            elems.end.val($.daterangeselector[target].range.end);
        }

        // Initialise the "selected" date values object
        $.daterangeselector[target].selected = new Object();
        $.daterangeselector[target].selected.start = $.daterangeselector[target].range.start;
        $.daterangeselector[target].selected.end = $.daterangeselector[target].range.end;
        $.daterangeselector[target].selected.clicks = 0;

        $.daterangeselector.cleanDateRange(target);
        $.daterangeselector.renderDateRange(elems, target);

        // Set the values of the hidden html elements to the start and end dates
        if($.daterangeselector[target].range.start) {
            elems.start.val($.daterangeselector[target].range.start);
        }
        if($.daterangeselector[target].range.end) {
            elems.end.val($.daterangeselector[target].range.end);
        }
    }

    // Initialise target listeners
    $.daterangeselector.initListeners = function(elems, target) {
        // Day click listener
        $('#' + target + ' .months td.day').live('click', function() {
            da = String($(this).html());
            if(da.length == 1) {
                da = "0" + da;
            }
            machine = $(this).parents('.monthContainer').find('.machine').html();
            machine = machine + "-" + da;
            if(moment(machine).isValid()) {
                clicks = $.daterangeselector[target].selected.clicks;
                mid = elems.popup.first('.months .monthContainer').find('.machine').html() + "-15";
                middle = moment(mid).add('months', 1).format("YYYY-MM-DD");
                if(clicks == 0) {
                    $.daterangeselector[target].selected.start = machine;
                    $.daterangeselector[target].selected.end = machine;
                    $.daterangeselector.renderDateRangeCalendars(elems, target, middle);
                    $.daterangeselector[target].reopen = true;
                } else if (clicks > 0) {
                    $.daterangeselector[target].selected.end = machine;
                    $.daterangeselector.renderDateRangeCalendars(elems, target, middle);
                    $.daterangeselector[target].reopen = true;
                }
                clicks++;
                if(clicks > 1) {
                    $.daterangeselector[target].selected.clicks = 0;
                } else {
                    $.daterangeselector[target].selected.clicks = clicks;
                }
            }
        });
        // Apply button click listener
        $('#' + target + ' .popup .apply').live('click', function() {
            $.daterangeselector[target].selected.clicks = 0;
            if(target) {
                start = moment($.daterangeselector[target].selected.start).unix();
                end = moment($.daterangeselector[target].selected.end).unix();
                if(start > end) {
                    elems.start.val($.daterangeselector[target].selected.end);
                    elems.end.val($.daterangeselector[target].selected.start);
                    $.daterangeselector[target].range.start = $.daterangeselector[target].selected.end;
                    $.daterangeselector[target].range.end = $.daterangeselector[target].selected.start;
                } else {
                    elems.start.val($.daterangeselector[target].selected.start);
                    elems.end.val($.daterangeselector[target].selected.end);
                    $.daterangeselector[target].range.start = $.daterangeselector[target].selected.start;
                    $.daterangeselector[target].range.end = $.daterangeselector[target].selected.end;
                }
            }
            elems.popup.toggle();
            $.daterangeselector.cleanDateRange(target);
            $.daterangeselector.renderDateRange(elems, target);
            elems.range.change();
        });
        // Previous month click listener
        $('#' + target + ' .popup .btn.previous-month').live('click', function() {
            if(target) {
                first = elems.popup.first('.months .monthContainer').find('.machine').html() + "-15";
                previous = moment(first).format("YYYY-MM-DD");
                $.daterangeselector.renderDateRangeCalendars(elems, target, previous);
                $.daterangeselector[target].reopen = true;
            }
        });
        // Next month click listener
        $('#' + target + ' .popup .btn.next-month').live('click', function() {
            if(target) {
                first = elems.popup.first('.months .monthContainer').find('.machine').html() + "-15";
                next = moment(first).add('months', 2).format("YYYY-MM-DD");
                $.daterangeselector.renderDateRangeCalendars(elems, target, next);
                $.daterangeselector[target].reopen = true;
            }
        });
        // Closed button
        $('#' + target + ' .popup .closed').live('click', function() {
            elems.popup.toggle();
        });
        // ESC Key hides popup
        $(document).keyup(function(e){
            switch(e.keyCode) {
                case 27:
                    elems.popup.css('display', 'none');
                    break;
            }
        });
        elems.range.live("click", function() {
            $.daterangeselector.renderDateRangeCalendars(elems, target);
            elems.popup.toggle();
        });
    }

    $.daterangeselector.renderDateRangeCalendars = function(elems, target, view) {
        // Clean html inside popup
        elems.popup.find('.content').html('');

        // Figure out which 3 months to view
        if(view) {
            st = view;
        } else if($.daterangeselector[target].selected.view) {
            st = $.daterangeselector[target].selected.view;
        } else {
            st = $.daterangeselector[target].selected.start;
        }

        a = st;
        b = st;

        // Set previous and next months dates
        prev = moment(a).subtract('months', 1).format("YYYY-MM-DD");
        next = moment(b).add('months', 1).format("YYYY-MM-DD");

        // Add base html into popup
        elems.popup.find('.content').append('<div><button class="btn previous-month" dateTarget="' + target + '"><i class="icon-arrow-left"></i></button><div class="months"></div><button class="btn next-month" dateTarget="' + target + '"><i class="icon-arrow-right"></i></button></div><div class="show"><div class="buttons"><button class="btn btn-primary btn-mini apply" dateTarget="' + target + '">Apply</button></div></div>');
        html = "";
        // Generate the actual months with dates in them
        html += $.daterangeselector.generateDateRangeMonth(prev);
        html += $.daterangeselector.generateDateRangeMonth(st);
        html += $.daterangeselector.generateDateRangeMonth(next);
        elems.popup.find(".months").append(html);

        // Render the selected range
        $.daterangeselector.renderSelected(elems, target);
    }

    // Renders the date range
    $.daterangeselector.renderDateRange = function(elems, target) {
        elems.range.val($.daterangeselector[target].clean.start + " - " + $.daterangeselector[target].clean.end);
    }

    // Sets the clean value to the daterange object
    $.daterangeselector.cleanDateRange = function(target) {
        var start = '';
        start = $.daterangeselector[target].range.start;
        var end = '';
        end = $.daterangeselector[target].range.end;

        $.daterangeselector[target].clean = new Object();
        $.daterangeselector[target].clean.start = moment(start).format('MMM Do, YYYY');
        $.daterangeselector[target].clean.end = moment(end).format('MMM Do, YYYY');
    }

    // Generates a month, uses handlebars for templating
    $.daterangeselector.generateDateRangeMonth = function(date) {
        // Gets array of date values
        arr = $.daterangeselector.getDateRangeMonthArray(date);
        // Generate the template
        var template = Handlebars.compile($('#daterangeselector_month_template').html());
        var html = template({'month' : moment(date).format('MMMM'), 'year' : moment(date).format('YYYY'), 'machine' : moment(date).format("YYYY") + "-" + moment(date).format("MM"), 'data' : arr});
        return html;
    }

    // Generates an array of date values
    $.daterangeselector.getDateRangeMonthArray = function(date) {
        arr = {};
        days = ['sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat'];
        weeks = ['one', 'two', 'three', 'four', 'five', 'six'];

        $.each(weeks, function(k,v) {
            $.each(days, function(l,m) {
                arr[v] = {}
                arr[v][m] = '';
            });
        });

        s = moment(date).date(1).format("YYYY-MM-DD");
        e = moment(date).add('months', 1).date(0).format("YYYY-MM-DD");

        firstDay = moment(s).format("ddd").toLowerCase();
        lastDay = moment(e).format("DD").toLowerCase();

        x = 0;
        y = 0;
        z = 0;
        iter = 0;
        $.each(weeks, function(a, b) {
            x = 0;
            $.each(days, function(c, d) {
                if(y == 0) {
                    if(firstDay == d) {
                        z = 1;
                        iter++;
                    }
                }
                if(iter) {    
                    arr[weeks[y]][days[x]] = z;
                    z++;
                }
                if(!z) {
                    arr[weeks[y]][days[x]] = '<div class="none"></div>';
                }
                if(lastDay < z) {
                    iter = 0;
                    z = 0;
                }
                x++;
            });
            y++;
        });
        return arr;
    }

    // Render the selected range
    $.daterangeselector.renderSelected = function(elems, target) {
        selected = new Object();
        selected.start = $.daterangeselector[target].selected.start;
        selected.end = $.daterangeselector[target].selected.end;

        st = moment(selected.start).unix();
        en = moment(selected.end).unix();
        tmp = 0;

        if(st > en) {
            tmp = st;
            st = en;
            en = tmp;
        }

        $('#' + target + ' .popup .months .monthContainer').each(function() {
            machine = $(this).find('.machine').html();
            $(this).find("td.day").each(function() {
                d = parseInt($(this).html());
                if(isInt(d)) {
                    d = String(d);
                    if(d.length == 1) {
                        d = "0" + d;
                    } else {
                        d = d;
                    }
                    cur = machine + "-" + d;
                    curDay = moment(cur).unix();
                    if(curDay >= st && curDay <= en) {
                        $(this).addClass("highlighted");
                    }
                }
            });
        });
    }
});

$(document).ready(function() {
    $('.date-range').each(function() {
        if($(this).attr('id')) {
            $.daterangeselector.init($(this).attr('id'));
        }
    });
});